Uses Crt, Dos, Drivers, UScreen, A320x200, Palette, LittleF;
Type
  HeaderData = Record
    ID : Array[1..8] Of Char;
    UnKnow1 : Word;
    Folder : Array[1..8] Of Char;
    Comment : Array[1..40] Of Char;
    UnKnow2 : Word;
    UnKnow3 : LongInt;
    Name : Array[1..8] Of Char;
    UnKnow4 : LongInt;
    FileSize : LongInt;
    UnKnow5 : LongInt;
    DataSize : Array[1..4] of Byte;
  End;

  FileHeader = Record
    ID : Array[1..4] Of Char;   {ID = CyBx}
    Comment : Array[1..32] Of Char;
    NLevel : Word;
    CurrentLevel : Word;
    MaxLevel : Word;
  End;

  LevelHeader = Record
    Comment : Array[1..20] Of Char;
    Data : Array[1..75] Of Byte;
  End;

Var
  Head : HeaderData;
  Level : Array[1..255, 1..15, 1..10] Of Byte;
  Comment : String[32];
  LevelComment : Array[1..255] of String[20];
  Sprites : Array[1..17, 1..18, 1..18] Of Byte;
  I : Word;
  Data : Byte;
  MyEvent : TEvent;
  Rep : Char;
  BPal : PaletteT;
  CurrentLeftBlock, CurrentRightBlock : Word;
  LevelHead : LevelHeader;
  FileHead : FileHeader;
  NLevel : Word;
  CurrentLevel : Word;
  AskL, FileName : String;
  NewFile : Word;

Procedure LoadSprite(Name : String);
Var
  F : File;
  X, Y : Word;
Begin
  Assign(F, Name);
  Reset(F, 1);
  Seek(F, 10);
  BlockRead(F, BPal, 768);
  LPalette(BPal);

  For I := 1 To 17 Do
    For Y := 1 To 18 Do
      For X := 1 To 18 Do
      Begin
        BlockRead(F, Data, 1);
        Sprites[I, X, Y] := Data;
      End;
  Close(F);
End;

Function ReadLevelFile(Name : String) : Boolean;
Var
  F : File;
  X, Y, CurrentData : Word;
Begin
  ReadLevelFile := True;
  Assign(F, Name);
  {$I-}
  Reset(F, 1);
  If IoResult <> 0 Then Begin ReadLevelFile := False; Exit; End;
  {$I+}
  Seek(F, 88);
  BlockRead(F, FileHead, SizeOf(FileHeader));

  If FileHead.ID <> 'CyBx' Then Begin ReadLevelFile := False; Exit; End;

  For I := 1 To 32 Do
    Comment[I] := FileHead.Comment[I];

  NLevel := FileHead.NLevel Shr 8;

  For I := 1 To NLevel Do
  Begin
    BlockRead(F, LevelHead, SizeOf(LevelHeader));
    For X := 1 To 20 Do
      LevelComment[I][X] := LevelHead.Comment[X];
    X := 1;
    Y := 1;
    CurrentData := 0;
    Repeat
      If CurrentData And 1 = 0 Then
        Level[I, X, Y] := LevelHead.Data[CurrentData Div 2 + 1] Shr 4
      Else
        Level[I, X, Y] := LevelHead.Data[CurrentData Div 2 + 1] And 15;

      Inc(CurrentData);
      Inc(X);
      If X > 15 Then
      Begin
        X := 1;
        Inc(Y);
      End;
    Until Y > 10;
  End;

  Close(F);
  ReadLevelFile := True;
End;

Procedure WriteLevelFile(Name : String);
Var
  F : File;
  CheckSum, X, Y : Word;
  CurrentData : Word;
  D : DirStr;
  N : NameStr;
  E : ExtStr;
Begin
  Assign(F, Name);
  Rewrite(F, 1);

  Head.ID := '**TI89**';
  Head.UnKnow1 := 1;
  Head.Folder := 'main' + Chr(0) + '   ';
  For I := 1 To 40 Do
    Head.Comment[I] := Chr(0);
  Head.UnKnow2 := 1;
  Head.UnKnow3 := 82;
  Head.UnKnow4 := 12;
  Head.FileSize := 0; {Pour apres}
  Head.UnKnow5 := 23205;
  Head.DataSize[1] := 0;
  Head.DataSize[2] := 0;
  Head.DataSize[3] := 0;
  Head.DataSize[4] := 0;

  FSplit(FileName, D, N, E);

  For I := 1 To 8 Do
    If I <= Length(N) Then
      Head.Name[I] := N[I] Else Head.Name[I] := Chr(0);

  BlockWrite(F, Head, SizeOf(HeaderData));

  FileHead.ID := 'CyBx';
  For I := 1 To 32 Do
    FileHead.Comment[I] := Comment[I];
  FileHead.NLevel := NLevel * 256;
  FileHead.CurrentLevel := 256;
  FileHead.MaxLevel := 256;

  BlockWrite(F, FileHead, SizeOf(FileHeader));

  For I := 1 To NLevel Do
  Begin
    LevelHead.Comment[1] := Chr(0);
    For X := 1 To 20 Do LevelHead.Comment[X] := LevelComment[I][X];
    X := 1;
    Y := 1;
    CurrentData := 0;
    FillChar(LevelHead.Data, 75, 0);
    Repeat
      If CurrentData And 1 = 0 Then
        LevelHead.Data[CurrentData Div 2 + 1] := Level[I, X, Y] * 16
      Else
        Inc(LevelHead.Data[CurrentData Div 2 + 1], Level[I, X, Y]);

      Inc(CurrentData);
      Inc(X);
      If X > 15 Then
      Begin
        X := 1;
        Inc(Y);
      End;
    Until Y > 10;
    BlockWrite(F, LevelHead, SizeOf(LevelHeader));
  End;

  Data := 0; {0 du string}
  BlockWrite(F, Data, 1);
  Data := 45; {Type string}
  BlockWrite(F, Data, 1);

  Seek(F, 0);
  Head.DataSize[3] := (FileSize(F) - SizeOf(HeaderData)) Shr 8;
  Head.DataSize[4] := (FileSize(F) - SizeOf(HeaderData)) And 255;
  BlockWrite(F, Head, SizeOf(HeaderData));

  {Calcule le checksum}
  CheckSum := 0;
  Seek(F, 86);
  For I := 86 To FileSize(F) - 1 Do
  Begin
    BlockRead(F, Data, 1);
    Inc(CheckSum, Data);
  End;

  Seek(F, FileSize(F));
  Data := CheckSum And 255;
  BlockWrite(F, Data, 1);
  Data := CheckSum Shr 8;
  BlockWrite(F, Data, 1);

  Head.FileSize := FileSize(F);
  Seek(F, 0);
  BlockWrite(F, Head, SizeOf(HeaderData));


  Close(F);
End;

Procedure DrawBlock(X, Y, SpriteType : Word);
Var
  VarX, VarY : Word;
Begin
  For VarY := 1 To 18 Do
    For VarX := 1 To 18 Do
    Begin
      Mem[$A000 : (X - 1) * 18 + VarX + (VarY + (Y - 1) * 18) * 320] :=
        Sprites[SpriteType, VarX, VarY];
    End;
End;

Procedure DrawLevel;
Var
  X, Y : Word;
  NLevelS : String;
Begin
  HideMouse;
  Str(CurrentLevel, NLevelS);
  BoxD(300, 1, 320, 6, 1);
  TransSentence('LEVEL ' + NLevelS, 13, 0, 275, 1, 500, 500, 500);
  Level[CurrentLevel, 8, 10] := 15;
  For Y := 1 To 10 Do
    For X := 1 To 15 Do
    Begin
      DrawBlock(X, Y, Level[CurrentLevel, X, Y] + 1);
    End;
  DrawBlock(8, 10, 16);
  ShowMouse;
End;

Procedure DrawChooseBlock;
Var
  X : Word;
Begin
  For X := 1 To 8 Do
  Begin
    DrawBlock(1, X, X);
    If X < 8 Then DrawBlock(3, X, 8 + X);
  End;
  TransSentence('CLICK ON THE BLOCK WITH THE LEFT OR THE RIGHT MOUSE BUTTON',
    14, 0, 50, 190, 500, 500, 500);
End;

Procedure ChooseBlock;
Begin
  HideMouse;
  ClsD(0);
  DrawChooseBlock;
  ShowMouse;
  Repeat
    GetMouseEvent(MyEvent);
    If MyEvent.Buttons > 0 Then
    Begin
      If MyEvent.Where.Y * 8 <= 140 Then
      Begin
        If MyEvent.Where.X * 4 <= 18 Then
        Begin
          If MyEvent.Buttons = 1 Then
            CurrentLeftBlock := (MyEvent.Where.Y * 8) Div 18
          Else
            CurrentRightBlock := (MyEvent.Where.Y * 8) Div 18;
          Exit;
        End;
        If (MyEvent.Where.X * 4 >= 36) And
             (MyEvent.Where.X * 4 <= 54) And
               (MyEvent.Where.Y * 8 <= 120) Then
        Begin
          If MyEvent.Buttons = 1 Then
            CurrentLeftBlock := (MyEvent.Where.Y * 8) Div 18 + 8
          Else
            CurrentRightBlock := (MyEvent.Where.Y * 8) Div 18 + 8;

          Exit;
        End;
      End;
    End;
    If KeyPressed Then
    Begin
      Rep := ReadKey;
    End;
  Until Rep = Chr(27);
End;

Procedure WriteCurrentBlock;
Begin
  HideMouse;
  TransSentence('LEFT BLOCK', 14, 0, 275, 10, 500, 500, 500);
  DrawBlock(17, 2, CurrentLeftBlock + 1);
  TransSentence('RIGHT BLOCK', 14, 0, 275, 47, 500, 500, 500);
  DrawBlock(17, 4, CurrentRightBlock + 1);

  TransSentence('F1 = SAVE FILE', 14, 0, 0, 181, 500, 500, 500);
  TransSentence('ESC = QUIT', 14, 0, 0, 188, 500, 500, 500);
  TransSentence('F2 = NEW LEVEL', 14, 0, 60, 181, 500, 500, 500);
  TransSentence('F3 = DEL LEVEL', 14, 0, 60, 188, 500, 500, 500);
  TransSentence('ENTER = CHOOSE BLOCK', 14, 0, 122, 181, 500, 500, 500);
  TransSentence('F4 = LEVEL COMMENT', 14, 0, 122, 188, 500, 500, 500);
  TransSentence('F5 = FILE COMMENT', 14, 0, 210, 181, 500, 500, 500);
  TransSentence('F6 = LEVEL INFO', 14, 0, 210, 188, 500, 500, 500);
  TransSentence('+ = LEVEL+1', 14, 0, 275, 120, 500, 500, 500);
  TransSentence('- = LEVEL-1', 14, 0, 275, 130, 500, 500, 500);
  ShowMouse;
End;

Procedure ChangeLevelComment;
Begin
  TextMode(3);
  AskL := Ask('Comment for level (19 char. max.)', All);

  For I := 1 To 20 Do
    LevelComment[CurrentLevel][I] := AskL[I];
  If Length(AskL) > 19 Then LevelComment[CurrentLevel][20] := Chr(0) Else
    LevelComment[CurrentLevel][length(AskL) + 1] := Chr(0);

  InitMode;
  LPalette(BPal);
  ClsD(0);
  WriteCurrentBlock;
  DrawLevel;
  ShowMouse;
End;

Procedure ChangeFileComment;
Begin
  TextMode(3);
  AskL := Ask('Comment for File (31 char. max.)', All);

  For I := 1 To 32 Do
    Comment[I] := AskL[I];
  If Length(AskL) >= 32 Then Comment[32] := Chr(0) Else
    Comment[Length(AskL) + 1] := Chr(0);


  InitMode;
  LPalette(BPal);
  ClsD(0);
  WriteCurrentBlock;
  DrawLevel;
  ShowMouse;
End;


Procedure NewLevel;
Begin
  If NLevel = 255 Then Exit;
  Inc(NLevel);
  FillChar(Level[NLevel], 150, 0);
  CurrentLevel := NLevel;
  ChangeLevelComment;
End;

Procedure DelLevel;
Begin
  If CurrentLevel <> NLevel Then
    For I := CurrentLevel To NLevel - 1 Do
      Move(Level[I + 1], Level[I], 150)
  Else
    Dec(CurrentLevel);
  Dec(NLevel);

  ClsD(0);
  WriteCurrentBlock;
  DrawLevel;
End;

Procedure DisplayInfo;
Var
  TmpStr : String;
Begin
  Set80x28;
  PutStr(34, 0, 'Level Info', 15, 4);
  PutStr(0, 5, 'Level number  : ', 15, 0);
  Str(CurrentLevel, TmpStr);
  PutStr(16, 5, TmpStr, 14, 0);
  PutStr(0, 6, 'Level comment : ', 15, 0);
  If LevelComment[CurrentLevel][1] <> Chr(0) Then
  Begin
    I := 1;
    TmpStr := '';
    Repeat
      TmpStr := TmpStr + LevelComment[CurrentLevel][I];
      Inc(I);
    Until (I > 19) Or (LevelComment[CurrentLevel][I] = Chr(0));
  End;
  If TmpStr = '' Then TmpStr := 'No Comment';
  PutStr(16, 6, TmpStr, 14, 0);

  Repeat Until KeyPressed;
  Rep := ReadKey;
  Rep := Chr(255);

  InitMode;
  LPalette(BPal);

  ClsD(0);
  WriteCurrentBlock;
  DrawLevel;
  ShowMouse;
End;

Function CheckFile(NameOfFile : String) : Boolean;
Var
  F : File;
Begin
  Assign(F, NameOfFile);
  {$I-}
  Reset(F);
  If IoResult <> 0 Then Begin CheckFile := False; Exit; End;
  {$I+}
  Close(F);
  CheckFile := True;
End;

Procedure CheckFiles;
Begin
  If CheckFile('Boxedit.fnt') = False Then
  Begin
    ClrScr;
    WriteLn('CyberBox Editor v1.0 by Philippe Gervaise aKa CandyMan');
    WriteLn('File "boxedit.fnt" not found !');
    Halt;
  End;
  If CheckFile('Blocs.sci') = False Then
  Begin
    ClrScr;
    WriteLn('CyberBox Editor v1.0 by Philippe Gervaise aKa CandyMan');
    WriteLn('File "blocs.sci" not found !');
    Halt;
  End;
End;

Var
  BlockX, BlockY : Word;
Begin
  CheckFiles;
  Car := Car2;

  ChoiceTitle := 'Load or Create Level';
  AddChoice('Load level');
  AddChoice('New level');
  NewFile := Choice;
  DoneChoice;
  If NewFile = 0 Then
  Begin
    ClrScr;
    WriteLn('CyberBox Editor v1.0 by Philippe Gervaise aKa CandyMan');
    Halt;
  End;

  Cls(176, 1, 7);
  PutStr(0, 0, '              CyberBox Editor v1.0 by Philippe Gervaise aKa CandyMan            ', 0, 7);

  If NewFile = 2 Then
  Begin
    FileName := Ask('Level filename (8 char max): ', Digit + Alpha);
    FileName := Copy(FileName, 1, 8) + '.89s';
    FillChar(Level, 150, 0);
    NLevel := 1;
    LevelComment[1] := '???';
  End Else
  Begin
    FileName := OpenFile('*.89s');
    If (FileName = '') Then
    Begin
      ClrScr;
      WriteLn('CyberBox Editor v1.0 by Philippe Gervaise aKa CandyMan');
      Halt;
    End;
  End;
  Asm
    Mov AX, 13h
    Int 10h
  End;

  ReadFont('Boxedit.fnt');
  CurrentLeftBlock := 0;
  CurrentRightBlock := 1;
  Comment := 'Created by CandyMan''s Editor';
  CurrentLevel := 1;
  LoadSprite('Blocs.sci');
  If NewFile <> 2 Then
    If Not(ReadLevelFile(FileName)) Then
    Begin
      ClrScr;
      WriteLn('CyberBox Editor v1.0 by Philippe Gervaise aKa CandyMan');
      WriteLn('Level file ' + Filename + ' not valid !');
      Halt;
    End;
  DrawLevel;
  WriteCurrentBlock;
  InitEvents;

  Repeat
    GetMouseEvent(MyEvent);
    If MyEvent.Buttons > 0 Then
    Begin
      BlockX := (MyEvent.Where.X * 4) Div 18 + 1;
      BlockY := (MyEvent.Where.Y * 8) Div 18 + 1;
      If (BlockX < 16) And (BlockY < 11) Then
      Begin
        If MyEvent.Buttons = 1 Then
          Level[CurrentLevel, BlockX, BlockY] := CurrentLeftBlock
        Else Level[CurrentLevel, BlockX, BlockY] := CurrentRightBlock;
      End;
      DrawLevel;
    End;
    If KeyPressed Then
    Begin
      Rep := ReadKey;
      If Rep = '+' Then If CurrentLevel < NLevel Then
      Begin
        Inc(CurrentLevel);
        DrawLevel;
      End;

      If Rep = '-' Then If CurrentLevel > 1 Then
      Begin
        Dec(CurrentLevel);
        DrawLevel;
      End;

      If Rep = Chr(13) Then
      Begin
        ChooseBlock;
        Rep := Chr(255);
        HideMouse;
        ClsD(0);
        ShowMouse;
        WriteCurrentBlock;
        DrawLevel;
      End;
      If Rep = Chr(0) Then
      Begin
        Rep := ReadKey;
        If Rep = Chr(59) Then
          WriteLevelFile(FileName);
        If Rep = Chr(60) Then
          NewLevel;
        If Rep = Chr(61) Then
          DelLevel;
        If Rep = Chr(62) Then
          ChangeLevelComment;
        If Rep = Chr(63) Then
          ChangeFileComment;
        If Rep = Chr(64) Then DisplayInfo;
      End;
    End;
  Until Rep = Chr(27);

  DoneEvents;
  Textmode(3);
  WriteLn('CyberBox Editor v1.0 by Philippe Gervaise aKa CandyMan');
End.